/////////////////////////////////////////////////////////////////////////////
// @doc DistObj
// @module DistObj.Java | Implementation of the <c DistObj> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: DistObj.java,v 1.4 2002/08/19 20:31:43 assefa Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: DistObj.java,v $
// Revision 1.4  2002/08/19 20:31:43  assefa
// Add ability to set the pixel size parameter from javascript.
//
// Revision 1.3  2002/08/12 22:57:02  assefa
// Resset the distance measure value at the end of the process.
// Bug 1153.
//
// Revision 1.2  2001/08/24 14:36:03  assefa
// Correct Bug 442 : mesuring problem.
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////


import java.applet.Applet;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Color;                      
import java.awt.Dimension;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.Polygon;
import java.applet.Applet;

import java.lang.Double;
import java.lang.Integer;

import java.lang.Math;

///////////////////////////////////////////////////////////////////////////////////////
public class DistObj extends PolyObj
{
    // Constants.
    final protected String kstrToken                = "%s";
    // Parameters.
    final protected String kstrPixelSizeParam       = "PIXEL_SIZE";
    final protected String kstrDistanceMsgParam     = "DIST_MSG";
    final protected String kstrDistPrecisionParam   = "DIST_PRECISION";
    
    
    protected int    m_nPrecision = 3;
    protected double m_fPixelSize = 0.0;
    protected double m_fCurrentDistance = 0.0;
    protected String m_strMsg = null;
    
    // User to put string in the status bar.
    protected Rosa2000 m_applet;

    public DistObj( Map parent, Rosa2000 applet )
    {
        super( parent, applet );
        m_applet = applet;
        readPararam();
    }

    protected void readPararam()
    {
        // Get the message.
        m_strMsg = m_applet.getParameter( kstrDistanceMsgParam );

        // Get the precision.
        String strPrecision = m_applet.getParameter( kstrDistPrecisionParam );
            
        // Get the pixel size.
        //String strPixelSize = m_applet.getParameter( kstrPixelSizeParam );
        double dfPixelSize = m_applet.GetPixelSize();
        if( dfPixelSize > 0)
        {
            try
            {
                if( strPrecision != null )
                    m_nPrecision = Integer.valueOf( strPrecision ).intValue();
                m_fPixelSize = dfPixelSize;//Double.valueOf( strPixelSize ).doubleValue();
            }
            catch( NumberFormatException e )
            {
                System.err.println( "Invalid value "  + strPrecision + " in the parameter " );
            }
        }
    }
    
    public String getSubmitValue()
    {
        String strRetVal = new String();
        return strRetVal;
    }


    public double getDistance( Point pt1, Point pt2 )
    {
        
        double dfDelta = Math.pow( pt2.x -pt1.x, 2 ) + Math.pow( pt2.y - pt1.y, 2 );
        return (double)(Math.sqrt( dfDelta ) * m_fPixelSize);
        
    }
    
    // return the value with the trunked decimal
    protected String getValueString( double fValue )
    {
        double dfTmp = Math.pow(10,  m_nPrecision) * fValue;
        dfTmp = Math.ceil(dfTmp);  
        double dfNewValue = dfTmp / Math.pow(10,  m_nPrecision);
        return Double.toString(dfNewValue);
    }
        
    public String getDistanceStringMsg( double fValue )
    {
        String strVal = new String();
        // Get the token to replace;    
        if( m_strMsg != null )
        {
            int nIndex = m_strMsg.indexOf(  kstrToken );
            if( nIndex != -1 )
            {
                // Retreive the first part of the string
                strVal = m_strMsg.substring(0, nIndex );
                // Adde the distance.
                strVal += getValueString( fValue );
                // retreive the string left.
                if( m_strMsg.length() > nIndex + 2 )
                    strVal += m_strMsg.substring( nIndex + 2 );
            }
            
        }
        else
            strVal =  getValueString( fValue );
        
        return strVal;
    }
    
    // mouse Moved
    public void mouseMoved (MouseEvent evt)
    {
        if( m_poly.npoints > 0 )
        {
            Point ptLastPt = new Point( m_poly.xpoints[ m_poly.npoints -1 ],
                                                    m_poly.ypoints[ m_poly.npoints -1 ] );
                
            double fDist = getDistance( ptLastPt, evt.getPoint() );
            double dist = fDist + m_fCurrentDistance;
            m_applet.showText(  getDistanceStringMsg( fDist + m_fCurrentDistance  ) );
        }
        super.mouseMoved( evt );
    }

    //////////////////////////////////////////////////////////////////////////
    // MouseListener implementation
    public void mousePressed (MouseEvent evt)
    {
        if( evt.getClickCount() == 2 )
        {
            endProcessing();
            m_fCurrentDistance = 0.0;
        }
        else
        {
        
        if( m_poly.npoints > 0 )
        {
            // Get the latest coordinates
            Point ptLastPt = new Point( m_poly.xpoints[ m_poly.npoints -1 ],
                                                    m_poly.ypoints[ m_poly.npoints -1 ] );
                
            m_fCurrentDistance += getDistance( ptLastPt, evt.getPoint() );
            m_applet.showText(  getDistanceStringMsg( m_fCurrentDistance ) );
        }
            
        super.mousePressed( evt );
            
        }
    }

    public void  destroyEvent()
    {
        // reset the status bar
        m_applet.showText( "" );
        m_fCurrentDistance = 0.0;
        super.destroyEvent();
    }
        
    /////////////////////////////////
    public void mouseReleased (MouseEvent evt)
    {
        
        if( m_poly.npoints > 0 )
        {
            // Get the latest coordinates
            Point ptLastPt = new Point( m_poly.xpoints[ m_poly.npoints -1 ],
                                                    m_poly.ypoints[ m_poly.npoints -1 ] );
                
            m_fCurrentDistance += getDistance( ptLastPt, evt.getPoint() );
            m_applet.showText(  getDistanceStringMsg( m_fCurrentDistance ) );
        }
            
        super.mouseReleased( evt );
    }


}
