/////////////////////////////////////////////////////////////////////////////
// @doc Map
// @module Map.Java | Implementation of the <c Map> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: Map.java,v 1.8 2002/08/08 15:37:56 sacha Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: Map.java,v $
// Revision 1.8  2002/08/08 15:37:56  sacha
// Put back toolba button when printing but not for Netscape. I still have a problem in netscape. So I just removed the toolbar in Netscape when printing.
//
// Revision 1.7  2000/08/18 14:23:17  assefa
// Use double buffering when drawing (rectangle, circle..) instead of XOR.
//
// Revision 1.6  2000/06/09 16:53:43  daniel
// Added bIgnoreSubmit param to startAction() to prevent infinite loop
//
// Revision 1.5  2000/05/12 04:14:00  daniel
// Fixed JSACTION for I.E. (JSObject.eval() must be called on a window object)
//
// Revision 1.4  2000/05/12 01:51:29  daniel
// Added TB_BUT_?_TARGET and TB_BUT_?_JSACTION
//
// Revision 1.3  2000/04/06 11:26:19  gparent
// Fixed a drawing bug using default selection
//
// Revision 1.2  2000/03/30 02:15:50  gparent
// Trap an exception when a parameter is missing inthe javascript
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////
// Import Modules.
import java.awt.Image;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;                               
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;                      
                                    
import netscape.javascript.*;
/**
 * Map is a class to manage the drawing map and the command apply on it. It receives
 * the command from the button and create the drawing object associated to the command.
 * At the end of operation, it submit the information if the command need it. 
 * 
 * @author Guy Parent
 */
public class Map  extends Component
   implements  ActionListener 
{
   // Constant command. /////////////////////////////////////////////////////////////////// 
   /** Command Rectangle */
   final private String kstrRect      = "rect";
   /** Command Point */
   final private String kstrPoint     = "point";
   /** Command Ellipse */
   final private String kstrEllipse   = "ellipse";
   /** Command Polygone */
   final private String kstrPoly      = "poly";
   /** Command Legend */
   final private String kstrLegend    = "legend";
   /** Command Distance */
   final private String kstrDistance  = "dist";
   /** Command Submit */
   final private String kstrSubmit    = "submit";
   
   /** Prefix to validate the auto submit command. */
   final private String kstrAutoMode  = "auto_";
   
   // Constant submit parameters./////////////////////////////////////////////////////////// 
   /** Form name parameter  */
   final private String kstrFormParam     = "INP_FORM_NAME";
   /** Type name parameter  */
   final private String kstrTypeParam     = "INP_TYPE_NAME";
   /** Coordinate name parameter    */
   final private String kstrCoordParam    = "INP_COORD_NAME";
   /** Location to draw the map     */
   final private String kstrMapCoordParam = "IMG_XY";
   
   
   // Attributes /////////////////////////////////////////////////////////////////////////////// 
   /** Image on the map     */
   private Image       m_ImagMap       = null;
   /** Graphic use for drawing  */
    //private Graphics    m_Graphics      = null;
   /** Current drawing object   ( command ) */
   private DrawObj     m_curDrawObj    = null;

    /** list of drawing objects **/
    private DrawObj     m_RectDrawObj    = null;
    private DrawObj     m_PointDrawObj    = null;
    private DrawObj     m_EllipseDrawObj    = null;
    private DrawObj     m_PolyDrawObj    = null;
    private DrawObj     m_LegendDrawObj    = null;
    private DrawObj     m_DistDrawObj    = null;

   /** Current button used.  */
   private ButtonCtrl  m_curButton     = null;
   /** Location of the map.     */
   private Point       m_coord         = null;
   /** Use to know if the current command was already used. */
   private boolean     m_bAlreadyUsed  = false;
   
   /** Keep the form name   come from parameter*/
   private String m_strFormName = null;
   /** Keep the type name   come from parameter*/
   private String m_strTypeName = null;
   /** Keep the coordinate name     come from parameter*/
   private String m_strCoordName = null;
   
   private Rosa2000 m_parent;

    protected Image       m_offScreenImage = null;
    protected Graphics 	m_offScreenGraphics = null;

   /** 
    * Constructor. Initialize the map.
    * 
    * @param applet Parent object
    * @param map Image to hadle.
    * @param nWidth width of the current map.
    * @param nHeight height of the current map. 
    */
   public Map ( Rosa2000 applet, Image map, int nWidth, int nHeight )
   {
      setVisible( false );
      m_ImagMap   = map;
      m_parent        = applet;
      setSize( nWidth, nHeight  );
      readSubmitParams( );
      readLocation();
      
      //create buffer (same size as applet) used for double buffering
      Dimension appletDim = m_parent.getSize();
      m_offScreenImage =  m_parent.createImage(appletDim.width, appletDim.height);
      m_offScreenGraphics = m_offScreenImage.getGraphics();
   }
   
   // Get/Set Methods
   /**
    * Set the current drawing object. This object take the control on the mouse event.
    * @param drawObj New drawing object.
    */
   public void setObjectDrawer( DrawObj drawObj )
   {
      m_curDrawObj = drawObj; 
   }

   /**
    * Return the current drawing object.
    */
   public DrawObj getObjectDrawer( )
   {
      return m_curDrawObj;    
   }

   /**
    * Return the Image of the map.
    * @see Image
    */
   public Image getImage()
   {
      return m_ImagMap;
   }
   
   /**
    * Return the Applet
    * @see Image
    */
   public Rosa2000 getApplet()
   {
      return m_parent;
   }
    
   /**
    * Check if the current command is in auto submit mode.
    * @return <b>true</b> if the command is prefixed by the string "auto_"
    */
   public boolean isAutoMode()
   {
      if( m_curButton != null )
      {
         String strCmd = m_curButton. getActionCommand();
         if( strCmd != null )
            return strCmd.startsWith( kstrAutoMode );
      }
      return false;
   }
   
   /**
    * Return the Graphics object used to draw the map.
    * @return Graphics object.
    */
   public Graphics getGraphics()
   {
      return m_offScreenGraphics;
      
   }
   
   /**
    * Paints this component. This method is called when the contents of the component 
    * should be painted in response to the component first being shown or damage needing repair. 
    * The clip rectangle in the Graphics parameter will be set to the area which needs to be painted. 
    * 
    * @param g The graphics context to use for painting.
    * @see Component#update
    */
   

   public void paint( Graphics g )
   {
       //if( m_Graphics == null )
       //m_Graphics = g.create();
       if (m_offScreenGraphics != null)
       {
           m_offScreenGraphics.drawImage(m_ImagMap, 0, 0, this); 
           //g.drawImage(m_ImagMap   , 0, 0, this);  
           if( getObjectDrawer() != null )
             getObjectDrawer().draw();
           g.drawImage(m_offScreenImage, 0, 0, this);
       }
   }

    /**
    * repaint the map.
    * 
    */
    public void redraw()
    {
        Graphics g = m_parent.getGraphics();
        if (g != null)
          paint(g);
    }



    /* this function is called by browser
       before printing the applet.*/
    public void printAll(Graphics g)
    {
	print(g);
    }

    public void print(Graphics g)
    {
	/* 
	   Netscape print the applet a little bigger than the
	   original component size. This cause netscape to print
	   the applet over some html component. To fix that, we
	   must resize the final result by 75%. This is documented
	   at this place: 

	   http://www.caff.de/workarounds/index.shtml#common_printing

	   If it is not netscape, use a scale factor of 100%
	*/
	double scale=0.75;

	// if not netscape set scale to 100%
	try 
	{
	    Class.forName("netscape.applet.MozillaAppletContext");
	} 
	catch (ClassNotFoundException e) 
	{ 
	    scale = 1.0;
	}
	
	if (m_offScreenGraphics != null)
	{
	    m_offScreenGraphics.drawImage(m_ImagMap, 0, 0, this); 
	    //g.drawImage(m_ImagMap   , 0, 0, this);  
	    if( getObjectDrawer() != null )
		getObjectDrawer().draw();
	    g.drawImage(m_offScreenImage, 0, 0, (int)(size().width*scale), (int)(size().height*scale), this);
	}
    }


    
    /**
    * Prepare and submit the operation.  This method is called when the DrawObj complet the
    * operation. It keep the javascript info and submit the information using form.  The submit
    * will be send only in auto mode or in sumbit command.  
    *  
    * This method use  <a href="http://home.netscape.com/eng/mozilla/3.0/handbook/plugins/pjava.html"> LiveConnect</a> technologies.
    */
   public void commitOperation()
   {
      if( m_strTypeName != null && m_strCoordName != null &&
          m_curButton.getButtonActionName() != null && m_curButton.getButtonActionValue() != null &&
          m_curButton.getActionCommand() != null )
      {
         try
         {
            // get the string submit
            JSObject window = JSObject.getWindow(m_parent);
            JSObject doc = (JSObject)window.getMember("document");
            JSObject form = null;
            
            if( m_strFormName != null )
               form = (JSObject)doc.getMember(m_strFormName);
            else
            {       /// Marche pas on verra plus tard,
               form = (JSObject)doc.getMember( "forms" );
               if( form != null )
                  form = (JSObject)form.getSlot(0);
            }
            
            // Set the type 
            if( form != null )
            {
               JSObject jsInputType = (JSObject)form.getMember(m_strTypeName);
               jsInputType.setMember("value",  m_curButton.getActionCommand() );
               
               // Set the coordinate
               if( m_curDrawObj != null )
               {
                  JSObject jsCoordinate = (JSObject)form.getMember(m_strCoordName);
                  jsCoordinate.setMember("value",  m_curDrawObj.getSubmitValue() );
               }
               
               // Set the action button
               JSObject jsButtonAction = (JSObject)form.getMember( m_curButton.getButtonActionName() );
               jsButtonAction.setMember("value",  m_curButton.getButtonActionValue() );

               // Submit
               if( isAutoMode() == true || m_curButton.getActionCommand().equals( kstrSubmit ) )
               {
                  // We either run the JSAction command, or submit the form (the default)
                  if (m_curButton.getJSAction() != null && m_curButton.getJSAction().length() != 0)
                  {
                     // Call a JavaScript method for this button instead of
                     // submitting.  The JavaScript code is responsible of
                     // submitting the form if necessary.
                     JSObject.getWindow(m_parent).eval( m_curButton.getJSAction() );

                     // Since the form won't be submitted and thus the applet
                     // won't be reloaded, then we have to reset the current
                     // action.
                     //startAction( m_curButton, true );
                     return;
                  }
                  else
                  {
                      // Default action: submit the form

                      // First apply submit target if set for this button
                      if (m_curButton.getTarget() != null && 
                          m_curButton.getTarget().length() != 0)
                      {
                          form.setMember("target", m_curButton.getTarget());
                      }

                      submitData( form );
                  }
               }
               else
                  m_curButton.setPressed( false );
            }
         }
         catch( java.lang.ClassCastException ex )
         {
            System.err.println( "A parameter is not found. Cannot submit to the server"); 
         }
      }
      else
      {
         // Reset the action.
         m_bAlreadyUsed = true;
         startAction( m_curButton, true );
         return;
      }
      
      m_bAlreadyUsed = true;
      // Destroy the command  an reset all.
//      if(  isAutoMode()==false || m_curButton != null  )
      //  destroyCommand();
   }

   /**
    * Destroy the current command.
    */
   protected void destroyCommand()
   {
      if( getObjectDrawer() != null )
         getObjectDrawer().destroyEvent();
      
      m_curDrawObj = null;
      m_curButton = null;
      
   }
   
   /** Submit all data put in the form objects.
    * @param form Live connect object used to sumit the data. All data should be
    * setted before call this method
    *
    */ 
   public void submitData( JSObject form )
   {
      Object[] args = new Object[0];
      form.call("submit", args);
   }
   
   /**
    * Called by the ButtonHandler when an operation start.
    * @param e ActionEvent send by the ButtonHandler
    */
   public void actionPerformed( ActionEvent e ) 
   {
      startAction( (ButtonCtrl)e.getSource(), false );
      
   }
   
   /**
    * Start the new operation associated to the button passed in parameter.
    * @param buttonPress Active button pressed by the user.
    * @param bIgnoreSubmit Pass true when setting default button to avoid 
    *                      infinite loop with "submit" buttons.
    */
   public void startAction( ButtonCtrl buttonPress,
                            boolean bIgnoreSubmit)
   {
      // Check if an object drawer is already used.
       if( getObjectDrawer() != null )
       {
           getObjectDrawer().disable_object();
       //destroyCommand();
       }
      
      if( m_bAlreadyUsed == true )
         repaint();

      if(buttonPress.isPressed() == true )
      {
         // Set the toolbar.
         m_parent.getTB().setButtonPress( buttonPress );
         
         String strName = buttonPress.toString();
         m_curButton = buttonPress;
         // Get the current Command
         String strCmd = m_curButton.getActionCommand();
         
         // Add the prefix Auto mode string.
         String strValidCmd = new String(); 
         if( isAutoMode() == true )
            strValidCmd = kstrAutoMode;
         
         if(strCmd == null)
           strCmd = "null";

         // Rectangle Command
         if( strCmd.equals( strValidCmd + kstrRect ) )
         {
             if (m_RectDrawObj == null)
               m_RectDrawObj = new RectObj( this );
             else
              m_RectDrawObj.enable_object();
            m_curDrawObj =  m_RectDrawObj;
         }// Point Command
         else if( strCmd.equals( strValidCmd + kstrPoint ) )
         {
             if (m_PointDrawObj == null)
               m_PointDrawObj = new PointObj( this );
             else
               m_PointDrawObj.enable_object();

             m_curDrawObj =   m_PointDrawObj;
         }// Ellipse Command
         else if( strCmd.equals( strValidCmd + kstrEllipse ) )
         {
             if (m_EllipseDrawObj == null)
               m_EllipseDrawObj = new EllipseObj( this );
             else
               m_EllipseDrawObj.enable_object();

             m_curDrawObj =  m_EllipseDrawObj;
            
         }// Polyline Command
         else if( strCmd.equals( strValidCmd + kstrPoly ) )
         {
             if (m_PolyDrawObj == null)
               m_PolyDrawObj = new PolyObj( this, m_parent );
             else
               m_PolyDrawObj.enable_object();

            m_curDrawObj =  m_PolyDrawObj;
         }// Legend Command
         else if( strCmd.equals( kstrLegend ) )
         {
             if (m_LegendDrawObj == null)
               m_LegendDrawObj = new LegendObj( this, m_parent );
             else
               m_LegendDrawObj.enable_object();

            m_curDrawObj =  m_LegendDrawObj;
         }// Distance and measurment
         else if( strCmd.equals( kstrDistance ) )
         {
             if (m_DistDrawObj == null)
               m_DistDrawObj = new DistObj( this, m_parent );
             else
               m_DistDrawObj.enable_object();

            m_curDrawObj =  m_DistDrawObj;
         }// Submit
         else if( strCmd.equals( kstrSubmit ) )
         {
            if (!bIgnoreSubmit)
               commitOperation();
         }
         else
         {   // Send an error... Command not supported
            System.err.println( "Command " + strCmd + " not supported"); 
         }
      }   
   }
   
   /////////////////////////////////////////////////////////////////////////////////////
   // Read Param
   
   /**
    * Read the location of the map.
    */
   protected void readLocation()
   { 
      String strLocation = m_parent.getParameter( kstrMapCoordParam );
      if( strLocation != null )
      {
         boolean bError = true;
         String strCoor = new String();
         int nX = 0, 
            nY = 0;
         
         int nIndex = 0;
         // Find the separator coordinate.
         if( ( nIndex = strLocation.indexOf(",", nIndex++ ) ) != -1 )
         {
            strCoor = strLocation.substring( 0, nIndex );
            // get the X value
            try
            {
               nX = Integer.valueOf( strCoor, 10 ).intValue();
               
               /// Get the Y value                 
               if( nIndex < strLocation.length()-1 )
               {
                  strCoor = strLocation.substring( nIndex+1 );

                  if( strCoor != null )
                  {
                     // get the number
                     nY = Integer.valueOf( strCoor, 10 ).intValue();
                     bError = false;
                  }
               }
            }
            catch( NumberFormatException e )
            {
               ;// do nothing
            }
         }
         
         // If no error, set the location.
         if( bError == false )
         {
            m_coord = new Point( nX, nY );
            setLocation ( m_coord );
         }
         else
         {
            System.err.println( "Invalid coordinate parameter: " + strLocation + " for the Map." );
         }
      }
      else
         centerMap();
   }

   /**
    * Center the map.
    */
   protected void centerMap()
   {
      
      Dimension mapSize = getSize();
      Dimension parentSize =  m_parent.getSize();
      
      m_coord = new Point( ( parentSize.width - mapSize.width )/2, 
                           ( parentSize.height - mapSize.height )/2    );
      
      setLocation( m_coord );

   }   
   
   
   /**
    *  Read Submit parameters.
    */
   protected void readSubmitParams( )
   {
      // Form Name
      m_strFormName   = m_parent.getParameter( kstrFormParam );
      
      // Type Name
      m_strTypeName       = m_parent.getParameter( kstrTypeParam );
      if( m_strTypeName == null )
         System.err.println( "The parameter " + kstrTypeParam + 
                             " is missing.  The submit command will be disabled."); 
      // Coordinate name
      m_strCoordName  = m_parent.getParameter( kstrCoordParam );
      if( m_strCoordName == null )
         System.err.println( "The parameter " + kstrCoordParam + 
                             " is missing.  The submit command will be disabled."); 
   }
   
}
