/////////////////////////////////////////////////////////////////////////////
// @doc SpacingButton
// @module SpacingButton.Java | Implementation of the <c SpacingButton> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: SpacingButton.java,v 1.2 2000/02/04 15:02:33 gparent Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: SpacingButton.java,v $
// Revision 1.2  2000/02/04 15:02:33  gparent
// Added doccumentation.
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////


import java.awt.Component;
import java.awt.Graphics;

/**
 * This class is a dummy button to simulate a break between two button in the
 * toolbar. This class do nothing. It's only take a space in the toolbar.  The parameter
 * to define this object looks like space_n where n is the number of pixel between 
 * the buttons.
 */
public class SpacingButton extends ButtonCtrl 
{
    /** Constant - Used to extract the space string parameters  */
    final static protected  String kstrPrafixNameParam  = "space_";
    
    /** Use to know the spacing orientation. If true, the spacing will be
     * take in y orientation.     */
    private boolean m_bVertical;
    
    /**
     * Contructor.
     * @param strSpacingName Name of the button.
     * @param Orientation.
     */
    SpacingButton( String strSpacingName, boolean bVertical )
    {
        super(  null, false, null  ) ;
        m_bVertical = bVertical;
        parseSize(  strSpacingName );
    }
    
    
    /**
     * Found the size in the parameters.
     */
    void parseSize(  String strSpacingName )
    {
        boolean bError = true;
        String strCoor = new String();
        int nVal = 0; 
             
            int nIndex = 0;
            // Check if the parameters name begin with the constant.
            if( strSpacingName.startsWith( kstrPrafixNameParam ) == true ) 
            { 
                try 
                { 
                    // Get the associated value.
                    String strValue = strSpacingName.substring( kstrPrafixNameParam.length() ); 
                    nVal = Integer.valueOf( strValue, 10 ).intValue(); 
                    // Set the vertical spacing.
                    if( m_bVertical == true )
                    { 
                        setSize( 1, nVal ); 
                        m_nXSize = 1; 
                        m_nYSize = nVal;
                    } 
                    else 
                    {
                        // Set the horizontal spacing
                        setSize( nVal, 1 );
                        m_nXSize = nVal; 
                        m_nYSize = 1;
                    } 
                } 
                catch( NumberFormatException e ) 
                {
                    ;// do nothing 
                } 
                catch( IndexOutOfBoundsException ndxException )
                {
                    System.err.println( "Spacing error" ); // send an error 
                }

            } 
             
            
    
    }
        
    /**
     * Paint nothing.
     */
    public void paint( Graphics g){}

    
}
